#!/bin/bash
#----------------------------------------------------------------------------
# Copyright (c) 2004-2023 Olaf Jaehrling <eisfair@ojaehrling.de>
#
# Creation:     13.03.2005 OJae
# Last Update:  15.04.2023 OJae
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License
#----------------------------------------------------------------------------
PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

if [ -f /etc/config.d/brute_force_blocking ]
then
    . /etc/config.d/brute_force_blocking
fi
if [ -f /etc/default/brute_force_blocking ]
then
    . /etc/default/brute_force_blocking
fi
if [ -f /usr/local/brute_force_blocking/configfile ]
then
    . /usr/local/brute_force_blocking/configfile
fi
if [ -f /var/install/include/eislib ]
then
    . /var/install/include/eislib
fi

work_path=${bfbhome}/
sqlite_bin=`which sqlite3`
DB_NAME=${work_path}/bfb.db
DB_TABLE=bfb_blocks
tmp_file=/tmp/bfb-free-cmd.$$
lastdate='2099-04-29'


if [ "$1" = "new" ]
then
    debug=yes
    echo "calculating BFB_FREE_IP_GROUP"
    echo "This may take a while, depending on how large the network is."
    echo "Berechne BFB_FREE_IP_GROUP"
    echo "Das kann eine Weile dauern, je nachdem, wie gross das Netzwerk ist."
    if [ "$BFB_USE_RAMDISK" = "yes" ]
    then
        bfbhome='/brute_force_blocking'
    fi
elif [ "$1" = "install" ]
then
    install=yes
fi
cd /usr/local/brute_force_blocking/leer

if [ "$BFB_BLOCK_REMOTE_SERVER" = "no" ]
then
    if [ "$BFB_USE_IPTABLES_NFTABLES" = "nftables" ]
    then
        NFT=`which nft`
        if ! $NFT list ruleset | grep -q "ip filter"
        then
            $NFT add table ip filter
            $NFT add chain ip filter INPUT "{ type filter hook input priority filter; policy accept; }"
        fi

        if ! $NFT list ruleset | grep -q "chain BFBBLOCK"
        then
            $NFT add chain ip filter BFBBLOCK
            $NFT add chain ip filter BOTBLOCK
            $NFT insert rule ip filter INPUT jump BFBBLOCK
            $NFT insert rule ip filter INPUT jump BOTBLOCK
        fi
    else
        IPTABLES=`which iptables`
        if ! $IPTABLES -nL BFBBLOCK >/dev/null 2>&1
        then
            $IPTABLES -N BFBBLOCK >/dev/null 2>&1
            $IPTABLES -N BOTBLOCK >/dev/null 2>&1
            $IPTABLES -I INPUT -j BFBBLOCK >/dev/null 2>&1
            $IPTABLES -I INPUT -j BOTBLOCK >/dev/null 2>&1
        fi
    fi
fi

#### free ip address
if [ "$BFB_FREE_IP" != "" ]
then
    while [ true ]
    do
        if [ "$debug" = "yes" ]
        then
            echo "adding free ip4 addresses"
        elif [ "$install" = "yes" ]
        then
            logger -t BFB -pnotice -is "adding free ip4 addresses"
        fi
        echo $BFB_FREE_IP |tr " " "\n" |while read ipaddr
        do
            if [ "$ipaddr" ]
            then 
                (
                echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', 'free',1)" 
                echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                ) >> ${tmp_file}
            fi
        done
        break
    done
fi
if [ "$BFB_BLOCK_USE_IPV6" = "yes" ]
then
    if [ "$BFB_FREE_IPV6" != "" ]
    then
        while [ true ]
        do
            if [ "$debug" = "yes" ]
            then
                echo "adding free ip6 addresses"
            elif [ "$install" = "yes" ]
            then
                logger -t BFB -pnotice -is "adding free ip6 addresses"
            fi
            echo $BFB_FREE_IPV6 |tr " " "\n" |while read ipaddr
            do
                if [ "$ipaddr" ]
                then
                    (
                    echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', 'free',1)" 
                    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                    ) >> ${tmp_file}
                fi
            done
            break
        done
    fi
    ipaddr='::1'
    (
    echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', 'free',1)" 
    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
    ) >> ${tmp_file}
fi

## Wenn temporear IP-Adressen in die "Free"-Liste eingetragen werden sollen,
## dann koenen die in die /usr/local/brute_force_blocking/tempfree eingetragen
## werden und werden hier beruecksichtigt
if [ -f /usr/local/brute_force_blocking/tempfree ]
then
    if [ -s /usr/local/brute_force_blocking/tempfree ]
    then
        while [ true ]
        do
            if [ "$debug" = "yes" ]
            then
                echo "adding free ip address/network tempfree"
            elif [ "$install" = "yes" ]
            then
                logger -t BFB -pnotice -is "adding free ip address/network tempfree"
            fi
            cat /usr/local/brute_force_blocking/tempfree |tr " " "\n" |while read ipaddr
            do
                if [ "$ipaddr" != "" ]
                then
                    (
                    echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', 'free',1)" 
                    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                    ) >> ${tmp_file}
                fi
            done
            break
        done
    fi
fi


#### free ip group

idx1=1
while [ "$idx1" -le "$BFB_FREE_IP_GROUP_N" ]
do
    eval free_ip_group='$BFB_FREE_IP_GROUP_'$idx1
    if [ "$debug" = "yes" ]
    then
        echo "adding free ip address/network $free_ip_group"
    elif [ "$install" = "yes" ]
    then
        logger -t BFB -pnotice -is "adding free ip address/network $free_ip_group"
    fi
    echo "$free_ip_group" > /tmp/zahlen
    DELIMITER=`echo $free_ip_group | sed "s/[0-9]//g" | sed "s/\.//g"`
    if [ "$DELIMITER" = "/" ]
    then
        if [ -f /var/install/packages/ipcalc ]
        then
            FIRSTIP=`ipcalc -n $free_ip_group | grep HostMin | awk '{print $2}'`
            LASTIP=`ipcalc -n $free_ip_group | grep HostMax | awk '{print $2}'`
            free_ip_group=`echo $FIRSTIP'-'$LASTIP`
            echo "$free_ip_group" > /tmp/zahlen
        elif [ -f /var/install/packages/netcalc-ng ]
        then
            FIRSTIP=`ipcalc -n $free_ip_group | grep HostMin | awk '{print $3}'`
            LASTIP=`ipcalc -n $free_ip_group | grep HostMax | awk '{print $3}'`
            free_ip_group=`echo $FIRSTIP'-'$LASTIP`
            echo "$free_ip_group" > /tmp/zahlen
        elif [ -f /usr/bin/ipcalc ]
        then
            FIRSTIP=`ipcalc -n $free_ip_group | grep HostMin | awk '{print $2}'`
            LASTIP=`ipcalc -n $free_ip_group | grep HostMax | awk '{print $2}'`
            free_ip_group=`echo $FIRSTIP'-'$LASTIP`
            echo "$free_ip_group" > /tmp/zahlen
        fi
    fi
    if [ "$free_ip_group" != "" ]
    then
        lo=`awk -F"-" '{print $1}' < /tmp/zahlen`
        hi=`awk -F"-" '{print $2}' < /tmp/zahlen`

        read a b c d <<< $(echo $lo |tr . ' ')
        read e f g h <<< $(echo $hi |tr . ' ')

        eval "echo {$a..$e}.{$b..$f}.{$c..$g}.{$d..$h}" | tr ' ' '\n' | \
        while read ipaddr
        do
            if [ "$ipaddr" != "" ]
            then
                (
                echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', 'free',1)"
                echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                ) >> ${tmp_file}
            fi
        done
    fi
    idx1=`/usr/bin/expr $idx1 + 1`
done

if [ -f ${tmp_file} ]
then
    if [ -f ${DB_NAME} ]
    then
        if [ "$debug" = "yes" ]
        then
            echo "import all free address/network into database"
        elif [ "$install" = "yes" ]
        then
            logger -t BFB -pnotice -is "import all free address/network into database"
        fi
        ${sqlite_bin} ${DB_NAME} < ${tmp_file}
    fi
fi

if [ "$BFB_USE_RAMDISK" = "yes" ]
then
    if [ -f $bfbhome/bfb.db ]
    then
        cp $bfbhome/bfb.db /usr/local/brute_force_blocking/bfb.db
    fi
fi
rm -f ${tmp_file}
rm -f /tmp/zahlen
cd - >/dev/null
exit 0
