#!/bin/bash
#----------------------------------------------------------------------------
# brute_force_blocking - edit /etc/config.d/brute_force_blocking and create new brute_force_blocking files
#
# Copyright (c) 2004-2023 Olaf Jaehrling <eisfair@ojaehrling.de>
#
# Creation:     13.03.2005 OJae
# Last Update:  30.03.2023 OJae
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License
#----------------------------------------------------------------------------
export PATH="/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:/bin:/sbin"
if [ -f /var/install/include/eislib ]
then
    . /var/install/include/eislib
fi
#exec 2>/tmp/B_F_b_sh_trace.log
#set -x

if [ -f /usr/local/brute_force_blocking/configfile ]
then
    . /usr/local/brute_force_blocking/configfile
fi
if [ -f /etc/config.d/brute_force_blocking ]
then
    . /etc/config.d/brute_force_blocking
fi

if [ -f /var/install/include/check-eisfair-version ]
then
    . /var/install/include/check-eisfair-version
fi

if [ "$BFB_USE_IPTABLES_NFTABLES" = "nftables" ]
then
    NFT=`which nft`
    NFTT=",`which nft`"
else
    IPTABLES="`which iptables` -w"
    IPTABLESS=`which iptables`
    IPTABLEST=",`which iptables`"
    if [ "$BFB_BLOCK_USE_IPV6" = "yes" ]
    then
        IP6TABLESS=",`which ip6tables`"
        IP6TABLES="`which ip6tables` -w"
        IP6TABLEST=",`which ip6tables`"
    fi
fi


if [ "$START_BRUTE_FORCE_BLOCKING" = "yes" ]
then
    echo "starting brute_force_blocking ...."
    mkdir -p /var/log/brute_force_blocking
    if [ -f /usr/local/brute_force_blocking/exclude.list ]
    then
        cat /usr/local/brute_force_blocking/exclude.list >/usr/local/brute_force_blocking/excludeliste
    fi
    if  [ -s /usr/local/brute_force_blocking/excludelist-cust.list ]
    then
        cat /usr/local/brute_force_blocking/excludelist-cust.list >>/usr/local/brute_force_blocking/excludeliste
    fi
    if [ "$monitor_mail" = "yes" -o  "$send_mail" = "yes" ]
    then
        SENDMAIL=`which sendmail 2>/dev/null`

        if [ "$SENDMAIL" = "" ]
        then
            echo "no mail package installed. Please install mail, ssmtp, msmtp or vmail!"
            echo "disable BFB_SEND_MAIL and BFB_MONITOR_MAIL."
            sed -i "/^BFB_SEND_MAIL=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
            sed -i "/^BFB_MONITOR_MAIL=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
        fi
    fi
    if [ "$BFB_BLOCK_REMOTE_SERVER" = "yes" ]
    then
        touch /usr/local/brute_force_blocking/remote
        INSTALLED=`eisman check putty-tools`
        if [ "$INSTALLED" != "installed" ]
        then
            eisman install putty-tools
        fi
        if [ "$BFB_BLOCK_REMOTE_SERVER_USE_OWN_COMMAND" = "no" ]
        then
            if [ "$BFB_BLOCK_REMOTE_SSH_KEY" = "" ]
            then 
                OPTION=''
            else
                OPTION='-i '$BFB_BLOCK_REMOTE_SSH_KEY
            fi
            if [ "$BFB_BLOCK_REMOTE_SERVER_PASSWD" = "" ]
            then
                PW=''
            else
                PW='-pw'
            fi
            if [ "$BFB_BLOCK_REMOTE_SSH_KEY" != "" ]
            then
                if [ "$BFB_BLOCK_REMOTE_SERVER_USER" = "root" ]
                then
                    HOME='/root'
                else
                    HOME='/home/'$BFB_BLOCK_REMOTE_SERVER_USER
                fi
                if [ ! -f $BFB_BLOCK_REMOTE_SSH_KEY ]
                then
                    if [ "$BFB_BLOCK_REMOTE_SERVER_PASSWD" = "" ]
                    then
                        echo "please press ENTER for the passphrase question"
                    else
                        echo "BFB requires a valid user name and password to access the"
                        echo "remote server. If puttygen asks you for a passphrase please"
                        echo "use the password "$BFB_BLOCK_REMOTE_SERVER_PASSWD
                    fi
                    echo "generate putty secretkey"
                    /usr/bin/puttygen -t rsa -C PUTTYKEY -o $BFB_BLOCK_REMOTE_SSH_KEY
                    echo "adding putty public key into $HOME/.ssh/authorized_keys"
                    /usr/bin/puttygen $BFB_BLOCK_REMOTE_SSH_KEY -O public-openssh >>$HOME/.ssh/authorized_keys
                fi
            fi
            mkdir -p /root/.putty/
            touch /root/.putty/sshhostkeys
            if ! grep -q $BFB_BLOCK_REMOTE_SERVER_IP /root/.putty/sshhostkeys
            then 
                echo $BFB_BLOCK_REMOTE_SERVER_IP "isn't in /root/.putty/sshhostkeys"
                if /var/install/bin/ask "Want to get it now add?" "yes"
                then
                      echo "You must now once enter 'y'"
                      PLINK=`which plink 2>/dev/null`
                      if [ "$BFB_BLOCK_REMOTE_SSH_KEY" = "" ]
                      then
                          OPTION=""
                      else
                          OPTION='-i '$BFB_BLOCK_REMOTE_SSH_KEY
                      fi
                      if [ "$BFB_BLOCK_REMOTE_SERVER_PASSWD" = "" ]
                      then
                          PW=''
                      else
                          PW='-pw'
                      fi
                      RPORT=$BFB_BLOCK_REMOTE_SERVER_PORT
                      RUSER=$BFB_BLOCK_REMOTE_SERVER_USER
                      RPASSWD=$BFB_BLOCK_REMOTE_SERVER_PASSWD
                      RIP=$BFB_BLOCK_REMOTE_SERVER_IP
                      $PLINK $OPTION -l $RUSER -pw $RPASSWD -P $RPORT $RIP exit
                else
                    echo "Bitte beachte. Du muss nach dieser Installation als user 'root' auf der Kommandozeile"
                    echo "plink "$OPTION "-l "$BFB_BLOCK_REMOTE_SERVER_USER" "$PW" "$BFB_BLOCK_REMOTE_SERVER_PASSWD "(all) REMOTE_SERVER"
                    echo "aufrufen und die Frage nach dem Cache mit 'y' beantworten. Ansonsten funktioniert das nicht!"
                    echo "Press ENTER to continue"
                    read a
                fi
            fi
        fi
        (
        echo "#!/bin/bash"
        echo "if [ -f /sbin/iptables ]"
        echo "then"
        echo '    IPTABLES="/sbin/iptables -w"'
        echo "else"
        echo '    IPTABLES="`which iptables` -w"'
        echo "fi"
        echo ""
        echo "### alle BFB werden geleert"
        echo '$IPTABLES -F BFBBLOCK >/dev/null 2>&1'
        echo '$IPTABLES -F BFBSCAN >/dev/null 2>&1'
        if [ "$BFB_MONITOR_BOT_ATTACK" = "yes" ]
        then
            echo '$IPTABLES -F BOTBLOCK >/dev/null 2>&1'
        fi
        if [ "$BFB_BLOCK_PROACTIVE_FROM_ATMA" = "yes" ]
        then
            echo '$IPTABLES -F BFBATMA >/dev/null 2>&1'
        fi
        echo ""
        echo "### nun wird gerueft ob es eine Chain $BFB_BLOCK_REMOTE_FORWARD_CHAIN gibt"
        echo "### Wenn ja, wird geprueft ob noch Eintraege vorhanden"
        echo "### sind. Wenn ja werden die geloescht"
        echo 'if $IPTABLES -nL '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' >/dev/null 2>&1'
        echo "then"
        echo '    $IPTABLES -nL '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' --line-numbers|grep -E "(BFB|BOTBLOCK)"|cut -c1 >/tmp/BFBZEILEN'
        echo '    ANZAHL1=`cat /tmp/BFBZEILEN |head -1`'
        echo "else"
        echo "    ANZAHL1=0"
        echo "fi"
        echo 'while [ "$ANZAHL1" -gt "0"  ]'
        echo "do"
        echo '    ZEILE=`head -n1 /tmp/BFBZEILEN`'
        echo '    if $IPTABLES -nL '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' --line-numbers|grep -E "(BFB|BOTBLOCK)"  >/dev/null 2>&1'
        echo '    then'
        echo '        $IPTABLES -D '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' $ZEILE'
        echo '        ANZAHL1=`cat /tmp/BFBZEILEN |head -1`'
        echo "    else"
        echo "        ANZAHL1=0"
        echo "    fi"
        echo "done"
        echo ""
        echo '    $IPTABLES -D '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' -j BOTBLOCK >/dev/null 2>&1'
        echo '    $IPTABLES -D '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' -j BFBBLOCK >/dev/null 2>&1'
        echo '    $IPTABLES -D '$BFB_BLOCK_REMOTE_FORWARD_CHAIN' -j BFBATMA >/dev/null 2>&1'
        echo "### nun wird gerueft ob es eine Chain INPUT gibt"
        echo "### Wenn ja, wird geprueft ob noch Eintraege vorhanden"
        echo "### sind. Wenn ja werden die geloescht"
        if [ "$BFB_BLOCK_REMOTE_FORWARD_CHAIN" != "INPUT" ]
        then
            echo 'if $IPTABLES -nL INPUT >/dev/null 2>&1'
            echo "then"
            echo '    $IPTABLES -nL INPUT --line-numbers|grep -E "(BFB|BOTBLOCK)"|cut -c1 >/tmp/BFBZEILEN'
            echo '    ANZAHL1=`cat /tmp/BFBZEILEN |head -1`'
            echo "else"
            echo "    ANZAHL1=0"
            echo "fi"
            echo 'while [ "$ANZAHL1" -gt "0"  ]'
            echo "do"
            echo '    ZEILE=`head -n1 /tmp/BFBZEILEN`'
            echo '    if $IPTABLES -nL INPUT --line-numbers|grep -E "(BFB|BOTBLOCK)"  >/dev/null 2>&1'
            echo '    then'
            echo '        $IPTABLES -D INPUT $ZEILE'
            echo '        ANZAHL1=`cat /tmp/BFBZEILEN |head -1`'
            echo "    else"
            echo "        ANZAHL1=0"
            echo '    fi'
            echo "done"
            echo '$IPTABLES -D INPUT -j BOTBLOCK >/dev/null 2>&1'
            echo '$IPTABLES -D INPUT -j BFBBLOCK >/dev/null 2>&1'
            echo '$IPTABLES -D INPUT -j BFBATMA >/dev/null 2>&1'

        fi    
        echo ""
        echo "### Nun werden die BFB eigenen Chains geloescht"
        echo '$IPTABLES -X BFBBLOCK >/dev/null 2>&1'
        echo '$IPTABLES -X BFBSCAN >/dev/null 2>&1'
        echo '$IPTABLES -X BOTBLOCK >/dev/null 2>&1'
        echo '$IPTABLES -X BFBATMA >/dev/null 2>&1'
        echo ""
        )>/usr/local/brute_force_blocking/remote_remove
    else
        rm -f /usr/local/brute_force_blocking/remote
    fi


    if [ "$send_curl" = "yes" ]
    then
        CURL=`which curl 2>/dev/null`
        if [ "$CURL" = "" ]
        then
            echo "curl package is missing"
            echo "please install curl"
            echo "disable BFB_SEND_VIA_CURL."
            sed -i "/^BFB_SEND_VIA_CURL=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
            echo "Press ENTER to continue"
            read a
        fi
    fi


    if [ "$send_twitter" = "yes" -o "$send_proactive" = "yes" ]
    then
        if [ ! -f /var/install/packages/ttytter ]
        then
            echo "package ttytter isn't installed"
            if /var/install/bin/ask "Do you install this now?" "yes"
            then
                eisman install ttytter
                clrhome
                echo
                if [ "$send_twitter" = "yes" ]
                then
                    if /var/install/bin/ask "Do you want to create the keyfile for your twitter account now?" "yes"
                    then
                        /var/install/bin/create_ttytter_account
                    else
                        echo "Please remember. You must create a ttytter keyfile for your"
                        echo "Twitter account. If you don't want to create this at the moment"
                        echo "call 'setup > Service administration > ttytter > Generate keyfile'"
                        echo "Press ENTER to continue"
                        read a
                    fi
                fi
            else
                echo "please don't forget to install this package"
                echo "It will disable BFB_SEND_TO_TWITTER."
                sed -i "/^BFB_SEND_TO_TWITTER=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
                sed -i "/^BFB_SEND_ATTACKER_TO_TWITTER=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
                echo "Press ENTER to continue"
                read a
            fi
        else
            if [ "$send_twitter" = "yes" ]
            then
                if [ ! -f /usr/local/ttytter/Keys/.ttytterkey_$twitteruser ]
                then
                    if /var/install/bin/ask "Do you want to create the keyfile for your twitter account now?" "yes"
                    then
                        /var/install/bin/create_ttytter_account
                    else
                        clrhome
                        echo
                        echo "Please remember. You must create a ttytter keyfile for your"
                        echo "Twitter account. If you don't want to create this at the moment"
                        echo "call 'setup > Service administration > ttytter > Generate keyfile'"
                        echo "Press ENTER to continue"
                        read a
                    fi
                fi
            fi
        fi
    fi


    if [ "$BFB_BLOCK_PROACTIVE_FROM_DATABASE" = "yes" -a "$BFB_USE_IPTABLES_NFTABLES" = "iptables" ]
    then
        if ! [[ `eisman check ipset` = "installed" ]]
        then
            echo "BFB_BLOCK_PROACTIVE_FROM_DATABASE and iptables need ipset"
            echo "install this automaticly"
            sleep 2
            eisman install --auto ipset
        fi
    fi
    if [ "$monitor_sn" = "yes" ]
    then
        . /etc/config.d/sn
        if grep -q SN_USER_N /etc/config.d/sn
        then
            if [ "$SN_USER_N" = "0" ]
            then
                 echo "SN monitor not running while SN_USER_N = 0. Please enable userlogin."
                 echo "SN Ueberwachung ist ausgeschaltet, da keine Authentifizierung eingeschaltet ist. Bitte aktiviere die Authentifizierung."
                 echo "Press ENTER to continue"
                 read a
            fi
        elif ! grep -q SN_USER_N /etc/config.d/sn
        then
            echo "no user in sn configfile. You cannot monitor SN."
            echo "please set in BFB config BFB_MONITOR_SN='no'"
            echo "Press ENTER to continue"
            read a
            /var/install/bin/edit /etc/config.d/brute_force_blocking
            if ! grep -q SN_USER_N /etc/config.d/sn
            then
                echo "Hellooooo no user in sn configfile. You cannot monitor SN!!!"
                echo "please set BFB_MONITOR_SN='no' in BFB configuration !!!!!"
                echo "Press ENTER to continue"
                read a
                /var/install/bin/edit /etc/config.d/brute_force_blocking
            fi
        fi
    fi

    if [ "$BFB_MONITOR_OWNCLOUD" = "yes" ]
    then
        OLOG=owncloud.log
    elif [ "$BFB_MONITOR_NEXTCLOUD" = "yes" ]
    then
        OLOG=nextcloud.log
    fi

    if [ "$BFB_MONITOR_OWNCLOUD" = "yes" -o "$BFB_MONITOR_NEXTCLOUD" = "yes" ]
    then
        if [ -f /etc/config.d/owncloud ]
        then
            . /etc/config.d/owncloud
            OCLOG=$OWNCLOUD_DATA_DIR/$OLOG | sed s'-data/data-data-'
        elif [ -f /etc/config.d/nextcloud ]
        then
            . /etc/config.d/nextcloud
            OCLOG=$NEXTCLOUD_DATA_DIR/$OLOG | sed s'-data/data-data-'
        else
            if [ "$BFB_MONITOR_OWNCLOUD" = "yes" ]
            then
                OCLOG=$BFB_OWNCLOUD_LOG_DIR/$OLOG
            elif [ "$BFB_MONITOR_NEXTCLOUD" = "yes" ]
            then
               OCLOG=$BFB_NEXTCLOUD_LOG_DIR/$OLOG
            fi
        fi
        if [ ! -f $OCLOG ]
        then
            echo "connot find $OCLOG. Please verify your setup"
            sleep 5
        fi
    fi

    if [ -f /etc/brute_force_blocking.cfg ]
    then
        . /etc/brute_force_blocking.cfg
        if [ "$monitor_portscan" = "yes" ]
        then
            if $IPTABLES -nL BFB  2>/dev/null | grep -qs PSCAN >/dev/null 2>&1
            then
                $IPTABLES -N BFB  >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --tcp-flags ALL FIN,URG,PSH -j BFB >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --syn -j BFB >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --tcp-flags ALL NONE  -j BFB >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --tcp-flags ALL FIN -j BFB >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --tcp-flags SYN,RST SYN,RST -j BFB >/dev/null 2>&1
                $IPTABLES -I INPUT -p tcp --tcp-flags SYN,FIN SYN,FIN -j BFB >/dev/null 2>&1
                $IPTABLES -I BFB -p udp  -m limit --limit 20/minute -j LOG --log-prefix "PSCAN " >/dev/null 2>&1
                idxpr=1
                while [ "$idxpr" -le "$portscan_ranges_n" ]
                do
                    eval portrange='BFB_PORTSCAN_RANGE_'$idxpr
                    $IPTABLES -I BFB -p tcp --dport $portrange -j LOG -m limit --limit 20/minute --limit-burst 20 --log-level $level --log-prefix "PSCAN "  >/dev/null 2>&1
                    idxpr=`/usr/bin/expr $idxpr + 1`
                done

            fi
            if [ "$BFB_BLOCK_USE_IPV6" = "yes" ]
            then
                if $IP6TABLES -nL BFB  2>/dev/null | grep -qs PSCAN >/dev/null 2>&1
                then
                    $IP6TABLES -N BFB  >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --tcp-flags ALL FIN,URG,PSH -j BFB >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --syn -j BFB >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --tcp-flags ALL NONE  -j BFB >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --tcp-flags ALL FIN -j BFB >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --tcp-flags SYN,RST SYN,RST -j BFB >/dev/null 2>&1
                    $IP6TABLES -I INPUT -p tcp --tcp-flags SYN,FIN SYN,FIN -j BFB >/dev/null 2>&1
                    $IP6TABLES -I BFB -p udp  -m limit --limit 20/minute -j LOG --log-prefix "PSCAN " >/dev/null 2>&1
                    idxpr=1
                    while [ "$idxpr" -le "$portscan_ranges_n" ]
                    do
                        eval portrange='BFB_PORTSCAN_RANGE_'$idxpr
                        $IP6TABLES -I BFB -p tcp --dport $portrange -j LOG -m limit --limit 20/minute --limit-burst 20 --log-level $level --log-prefix "PSCAN "  >/dev/null 2>&1
                        idxpr=`/usr/bin/expr $idxpr + 1`
                    done
                fi
            fi
        fi
        if [ "$BFB_USE_IPTABLES_NFTABLES" != "nftables" ]
        then
            if $IPTABLES -nL FORWARD 2>/dev/null | grep -qs recent >/dev/null 2>&1
            then
              $IPTABLES -D FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT >/dev/null 2>&1
              $IPTABLES -D FORWARD -m recent --update -j DROP >/dev/null 2>&1
              idxp=1
              while [ "$idxp" -le "$specie_ports_n" ]
              do
                  eval port='$BFB_SPECIFIC_'$idxp'_PORTS'
                  $IPTABLES -D FORWARD -i $ext_interface -p tcp --dport $port -m recent --set -j DROP >/dev/null 2>&1
                  idxp=`/usr/bin/expr $idxp + 1`
              done
            fi
        fi
        if [ "$BFB_BLOCK_USE_IPV6" = "yes" ]
        then
            if $IP6TABLES -nL FORWARD 2>/dev/null | grep -qs recent >/dev/null 2>&1
            then
                $IP6TABLES -D FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT >/dev/null 2>&1
                $IP6TABLES -D FORWARD -m recent --update -j DROP >/dev/null 2>&1
                idxp=1
                while [ "$idxp" -le "$specie_ports_n" ]
                do
                    eval port='$BFB_SPECIFIC_'$idxp'_PORTS'
                    $IP6TABLES -D FORWARD -i $ext_interface -p tcp --dport $port -m recent --set -j DROP >/dev/null 2>&1
                    idxp=`/usr/bin/expr $idxp + 1`
                done
            fi
        fi
        rm -f  /etc/brute_force_blocking.cfg
    fi


    if [ "$monitor_ftp" = "yes" ]
    then
        if [ -f /etc/config.d/pure-ftpd ]
        then
            ARG=`grep PURE_FTPD_ADD_ARGS /etc/config.d/pure-ftpd | grep -e "\-H"`
            FTPPAKET=PUREFTP
        elif [ -f /etc/config.d/proftpd ]
        then
            ARG='dummy'
            FTPPAKET=PROFTP
        else
            ARG=""
        fi  

        if [ "$ARG" = "" ]
        then
            if [ "$FTPPAKET" = "PUREFTP" ]
            then 
                echo "To monitor FTPD correctly, you have to enter an additional tag for FTPD to the pure-ftp package configuration."
                echo "Please set 'PURE_FTPD_ADD_ARGS =  -H'"
                echo
                echo "Wenn die Ueberwachung des FTPD korrekt funktionieren soll, muss in der Config vom pure-Paket die zusaetzliche Variable"
                echo "'PURE_FTPD_ADD_ARGS =  -H' gesetzt werden!"
            elif [ "$FTPPAKET" = "" ]
            then
                echo "To monitor FTPD you must install pure-ftpd or pro-ftpd package"
                echo
                echo "Wenn die Ueberwachung des FTPD korrekt funktionieren soll musst du natuerlich auch prure-ftp oder pro-ftpd installieren."
                sed -i "/^BFB_MONITOR_FTPD=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
            fi
        fi
    fi


    if [ "$BFB_BLOCK_PROACTIVE_FROM_ATMA" = "yes" ]
    then
        if [ "$eisfairsystem" != "noneis" ]
        then
            if [ ! `which lynx 2>/dev/null` ]
            then
                if [ ! `which links 2>/dev/null` ]
                then
                    if [ ! `which curl 2>/dev/null` ]
                    then
                        PAKET=0
                        echo "no browser package installed"
                        echo "Do you want to install it now?"
                        echo "Please select:"
                        echo "1: links"
                        echo "2: lynx"
                        echo "3: curl"
                        count=1
                        while [ ${count} -le 3 ]
                        do
                            read PAKET
                            stty echo
                            echo ""
                            if [ "$PAKET" = "curl" ]
                            then
                                PAKET=curl
                            fi
                            if [ "$PAKET" = "0" -o "$PAKET" = "1" -o "$PAKET" = "2" -o "$PAKET" = "3" ]
                            then
                                if [ "$PAKET" = "1" ]
                                then
                                    PAKET=links
                                elif [ "$PAKET" = "2" ]
                                then
                                    PAKET=lynx
                                elif [ "$PAKET" = "3" ]
                                then
                                    PAKET=curl
                                elif [ "$PAKET" = "0" -o "$PAKET" = "" ]
                                then
                                    echo "You did not make a choice. BFB_BLOCK_PROACTIVE_FROM_ATMA will be deactivated."
                                    sed -i "/^BFB_BLOCK_PROACTIVE_FROM_ATMA=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
                                    break
                                fi
                                eisman install $PAKET
                                break
                                if [ "$PAKET" = "lynx" ]
                                then
                                    if [ ! -f /usr/local/ssl/certs/twitter_com.pem ]
                                    then
                                        cp /usr/local/brute_force_blocking/twitter-certs/twitter_com.pem /usr/local/ssl/certs/
                                    fi
                                    if [ ! -f /usr/local/ssl/certs/DigiCertSHA2HighAssuranceServerCA.crt.pem ]
                                    then
                                        cp /usr/local/brute_force_blocking/twitter-certs/DigiCertSHA2HighAssuranceServerCA.crt.pem /usr/local/ssl/certs/
                                    fi
                                    echo "copy Twitter certificate to /usr/local/ssl/certs/ and rehash."
                                    sleep 2
                                    /var/install/bin/certs-update-hashes
                                fi
                            else
                                echo "You must select 1,2, 3 or 0"
                                echo ""
                            fi
                            count=`expr ${count} + 1`
                        done
                    fi
                fi
            fi
        else
            if [ ! `which lynx 2>/dev/null` ]
            then
                if [ ! `which links 2>/dev/null` ]
                then
                    if [ ! `which curl 2>/dev/null` ] 
                    then 
                        echo "Don't forget to install links, curl or lynx package!"
                        sleep 10
                    fi
                fi
            fi
        fi
    fi


    (
    echo '. /etc/config.d/brute_force_blocking'
    echo '. /usr/local/brute_force_blocking/configfile'
    echo ''
    echo '$IPTABLES -F BFBBLOCK >/dev/null 2>&1'
#    echo 'sh /var/install/config.d/brute_force_blocking2.sh new'
    if [ "$BFB_FREE_DYNDNS_DOMAIN" = "yes" ]
    then
         echo '/usr/local/brute_force_blocking/dyndnsupdate.sh new'
    fi
    echo 'sleep 2'
    echo 'if [ "$1" != "stop" ]'
    echo 'then'
    echo '    if [ "$BFB_BLOCK_PROACTIVE_FROM_ATMA" = "yes" ]'
    echo '    then'
    echo '        sh /usr/local/brute_force_blocking/atma_es_update >/dev/null 2>&1'
    echo '    fi'
    echo 'fi'
    if [ "$BFB_SERVER_RUN_AS_SYSLOGSERVER" = "yes" -a "$BFB_BLOCK_REMOTE_SERVER" = "yes" ]
    then
        if [ "$BFB_BLOCK_REMOTE_SERVER_USE_OWN_COMMAND" = "no" ]
        then
            PLINK=`which plink 2>/dev/null`
            if [ "$BFB_BLOCK_REMOTE_SERVER_PASSWD" = "" ]
            then
                PW=''
            else
                PW='-pw'
            fi
            if [ "$BFB_BLOCK_REMOTE_SSH_KEY" != "" ]
            then
                OPTION='-i '$BFB_BLOCK_REMOTE_SSH_KEY
            else
                OPTION=''
            fi
            echo "touch /tmp/BFB.remote_leer"
            echo "while [ true ]"
            echo "do"
            echo '    ls /tmp/BFB.remote* | grep -v leer| awk -F"remote." '\'{print' $2}'\'' | sed s"/.tmp//" | sed s"/tmp//" |while read SERVERIP >/dev/null 2>&1'
            echo "    do"
            echo '        if [ "$SERVERIP" != "" ]'
            echo "        then"
            echo "            echo exit >> /tmp/BFB.remote.\$SERVERIP.tmp"
            echo "            $PLINK $OPTION -l $BFB_BLOCK_REMOTE_SERVER_USER $PW $BFB_BLOCK_REMOTE_SERVER_PASSWD "\$SERVERIP "</tmp/BFB.remote.\$SERVERIP.tmp"
            echo "            rm -f /tmp/BFB.remote.\$SERVERIP.tmp"
            echo "        fi"
            echo "    done"
            echo "    break"
            echo "done"
        else
            echo "\$BFB_BLOCK_REMOTE_SERVER_OWN_COMMAND \$ip"
        fi
    fi
    ) >/usr/local/brute_force_blocking/unblock


    chmod 755 /usr/local/brute_force_blocking/unblock
    idx1=1
    while [ "$idx1" -le "$BFB_FREE_IP_GROUP_N" ]
    do
        eval free_ip_group='$BFB_FREE_IP_GROUP_'$idx1
        #DELIMITER=`echo $free_ip_group | sed "s/[0-9]//g" | sed "s/\.//g"`
        #if [ "$DELIMITER" = "/" ]
        #then
        #    if [ ! -f /var/install/packages/ipcalc  ]
        #    then
        #        echo "The variable BFB_FREE_GROUP need the package ipcalc"
        #        sleep 2
        #        eisman install --unstable ipcalc
        #    fi
        #fi
        idx1=`/usr/bin/expr $idx1 + 1`
    done

    /usr/local/brute_force_blocking/brute_force_blocking2.sh 
    /usr/local/brute_force_blocking/dyndnsupdate.sh
    sleep 2

    if [ "$request_via_webserver" = "yes" ]
    then
        echo "create website files and modify sudoers ..."
        if [ "$apache_version" = "" ]
        then
            echo "BFB_USE_APACHE_VERSION is empty."
            echo "please configure the apache version 1 or 2 in config file."
            echo "activation aborted !!"
            echo "Press ENTER to continue"
            read a
            exit 1
        fi
        APACHEINSTALL='yes'
        if [ "$eisfairsystem" = "eisfair-1" ]
        then
            if [ ! -f /var/install/packages/apache -a ! -f /var/install/packages/apache2 ]
            then
                APACHEINSTALL='no'
            else
                if [ "$BFB_USE_IPTABLES_NFTABLES" = "nftables" ]
                then
                     (
                    echo "# added by brute_force_blocking"
                    echo "wwwrun ALL = NOPASSWD: /usr/local/brute_force_blocking/blocked $NFTT"
                    )>/etc/sudoers.d/brute_force_blocking
                    chmod 0440 /etc/sudoers.d/brute_force_blocking
                else
                    (
                    echo "# added by brute_force_blocking"
                    echo "wwwrun ALL = NOPASSWD: /usr/local/brute_force_blocking/blocked $IPTABLEST $IP6TABLEST"
                    )>/etc/sudoers.d/brute_force_blocking
                    chmod 0440 /etc/sudoers.d/brute_force_blocking
                fi
            fi
        elif [ "$eisfairsystem" = "eisfair-64" ]
        then
            if [ ! -f /var/install/packages/apache -a ! -f /var/install/packages/apache2 ]
            then
                APACHEINSTALL='no'
            else
                if [ "$BFB_USE_IPTABLES_NFTABLES" = "nftables" ]
                then
                     (
                    echo "# added by brute_force_blocking"
                    echo "wwwrun ALL = NOPASSWD: /usr/local/brute_force_blocking/blocked $NFTT"
                    )>/etc/sudoers.d/brute_force_blocking
                    chmod 0440 /etc/sudoers.d/brute_force_blocking
                else
                    (
                    echo "# added by brute_force_blocking"
                    echo "wwwrun ALL = NOPASSWD: /usr/local/brute_force_blocking/blocked $IPTABLEST $IP6TABLEST"
                    )>/etc/sudoers.d/brute_force_blocking
                    chmod 0440 /etc/sudoers.d/brute_force_blocking
                fi
            fi
        elif [ "$eisfairsystem" = "noneis" ]
        then
            if ! grep -q brute_force_blocking/blocked /etc/sudoers
            then
                (
                echo "# added by brute_force_blocking"
                echo "www-data ALL = NOPASSWD: /usr/local/brute_force_blocking/blocked $IPTABLEST $IP6TABLEST $NFTT"
                )>>/etc/sudoers
            fi
        fi
        if [ "$APACHEINSTALL" = "yes" ]
        then
            if [ ! -d $apache_documentroot ]
            then
                if [ -f /var/install/packages/apache -a -f /var/install/packages/apache2 ]
                then
                    mkdir -p $apache_documentroot
                    if [ ! -d $cgi_dokumentroot ]
                    then
                        mkdir -p $cgi_dokumentroot
                    fi
                else
                    echo "Are you sure Apache is installed? Please check this. Terminate brute_force_blocking setup now!"
                    APACHEINSTALL='no'
                    exit 1
                fi
            fi
        fi
        if [ "$APACHEINSTALL" = "yes" ]
        then
            bash /usr/local/brute_force_blocking/brute_force_blocking_web >/dev/null
        fi
        echo "done"
    fi


    if [ "$web" = "yes" -a "$eisfairsystem" != "noneis" ]
    then
        if [ "$apache_version" = "1" ]
        then
            echo "nutze apache1" >/dev/null
            if [ -f /etc/config.d/apache ]
            then
                . /etc/config.d/apache
                apache=$""
                vhost_n=$APACHE_VHOST_N
                apache_ssl=$APACHE_SSL
                apachelog=$APACHE_ERROR_LOG
                ssldir=$APACHE_SSL_LOGDIR
                ssllog1=$APACHE_ERROR_LOG".ssl"
            fi
        else
            if [ -f /etc/config.d/apache2 ]
            then
                . /etc/config.d/apache2
                if [ -f /etc/config.d/apache2_vhost ]
                then
                    . /etc/config.d/apache2_vhost
                fi
                apache=$"2"
                vhost_n=$APACHE2_VHOST_N
                apache_ssl=$APACHE2_SSL
                apachelog=$APACHE2_ERROR_LOG
                ssldir=$APACHE2_SSL_LOGDIR
                ssllog1=$APACHE2_ERROR_LOG".ssl"
            fi
        fi
        if [ ! -f /etc/config.d/apache ]
        then
            if [ ! -f /etc/config.d/apache2 ]
            then
                echo "error in configuration file. You want to monitor your webserver, but you havn't installed Apache"
                echo "Please set BFB_MONITOR_WEBACCESS=no or install Apache"
                echo "BFB_MONITOR_WEBACCESS will be deactivated."
                sed -i "/^BFB_MONITOR_WEBACCESS=/s/=.*/='no'/" /etc/config.d/brute_force_blocking
            fi
        fi

    elif [ "$eisfairsystem" = "noneis" ]
    then
        echo "Please don't forget to configure /etc/config.d/apache2 for the correct logfiles."
        sleep 5
    fi

    if [ "$monitor_mini_http" = "yes" ]
    then
        if [ -f /var/install/packages/mini_httpd ]
        then
            . /etc/config.d/mini_httpd
            if grep -q MINI_HTTPD_LOGFILE /etc/config.d/mini_httpd
            then
                if [ "$MINI_HTTPD_LOGFILE" = "" -o "$MINI_HTTPD_LOGFILE" = "none" ]
                then
                    echo "cannot find mini-http-log file or logfile is 'none'"
                    echo "please check mini-httpd config in your setup"
                    echo "Press ENTER to continue"
                    read a
                fi
            elif grep -q MINI_HTTPD_LOG_FILE /etc/config.d/mini_httpd
            then
                if [ "$MINI_HTTPD_LOG_FILE" = "" -o "$MINI_HTTPD_LOG_FILE" = "none" ]
                then
                    echo "cannot find mini-http-log file or logfile is 'none'"
                    echo "please check mini-httpd config in your setup"
                    echo "Press ENTER to continue"
                    read a
                fi
            fi
        else
            echo "are you sure you have installed mini_http?"
            echo "please install or reinstall mini_http."
            echo "Press ENTER to continue"
            read a
        fi
    fi

    if [ "$website" = "yes" ]
    then
        mkdir -p $apache_documentroot'/'$index_folder
        if [ -f /usr/local/brute_force_blocking/createiphtml ]
        then
            chmod +x /usr/local/brute_force_blocking/createiphtml
        fi
    fi
    if [ -f /etc/eisfair-system ]
    then
        service enable brute_force_blocking
    else
        systemctl enable brute_force_blocking.timer
    fi

else
    if [ -f /etc/eisfair-system ]
    then
        service stop brute_force_blocking
        service disable brute_force_blocking
    else
        systemctl disable brute_force_blocking.timer
        service brute_force_blocking stop
    fi
fi

exit 0
